import type { Document } from "dbtype/api";
import { Card, CardContent, CardDescription, CardHeader, CardTitle } from "@/components/ui/card";
import TagBadge from "@/components/gallery/TagBadge";
import { useEffect, useRef, useState } from "react";
import { Link, useLocation } from "wouter";
import { LazyImage } from "./LazyImage";

function clipTagsWhenOverflow(tags: string[], limit: number) {
    let l = 0;
    for (let i = 0; i < tags.length; i++) {
        l += tags[i].length;
        if (l > limit) {
            return tags.slice(0, i);
        }
        l += 1; // for space
    }
    return tags;
}

export function GalleryCard({
    doc: x
}: { doc: Document; }) {
    const ref = useRef<HTMLDivElement>(null);
    const [clipCharCount, setClipCharCount] = useState(200);
    const [location] = useLocation();

    const artists = x.tags.filter(x => x.startsWith("artist:")).map(x => x.replace("artist:", ""));
    const groups = x.tags.filter(x => x.startsWith("group:")).map(x => x.replace("group:", ""));

    const originalTags = x.tags.map(x => x.replace("artist:", "").replace("group:", ""));
    const clippedTags = clipTagsWhenOverflow(originalTags, clipCharCount);

    useEffect(() => {
        const listener = () => {
            if (ref.current) {
                const { width } = ref.current.getBoundingClientRect();
                const charWidth = 8; // rough estimate
                const newClipCharCount = Math.floor(width / charWidth) * 3;
                setClipCharCount(newClipCharCount);
            }
        };
        window.addEventListener("resize", listener);
        return () => {
            window.removeEventListener("resize", listener);
        };
    }, []);

    return <Card className="flex h-[200px]">
        <div className="rounded-xl overflow-hidden h-[200px] w-[142px] flex-none bg-[#272733] flex items-center justify-center">
            <LazyImage src={`/api/doc/${x.id}/comic/thumbnail`}
                alt={x.title}
                className="max-h-full max-w-full object-cover object-center"
            />
        </div>
        <div className="flex-1 flex flex-col">
            <CardHeader className="flex-none">
                <CardTitle>
                    <Link href={`/doc/${x.id}`} state={{fromUrl: location}}>{x.title}</Link>
                </CardTitle>
                <CardDescription>
                    {artists.join(", ")} {groups.length > 0 && "|"} {groups.join(", ")}
                </CardDescription>
            </CardHeader>
            <CardContent className="flex-1" ref={ref}>
                <ul className="flex flex-wrap gap-2 items-baseline content-start">
                    {clippedTags.map(tag => <TagBadge key={tag} tagname={tag} className="" />)}
                    {clippedTags.length < originalTags.length && <TagBadge tagname="..." className="" disabled />}
                </ul>
            </CardContent>
        </div>
    </Card>;
}
